Imports System.Drawing.Drawing2D

Public Class Form1
    ' The circle parameters.
    Private Circles As New List(Of RectangleF)()

    ' Variables to get the newest circle.
    Private X1, Y1, X2, Y2 As Integer
    Private Drawing As Boolean = False

    ' The points of intersection.
    Private Intersection1, Intersection2 As PointF
    Private NumIntersections As Integer = 0

    ' Get a circle.
    Private Sub Form1_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        If (Circles.Count = 2) Then
            ' Start over.
            Circles.Clear()
            NumIntersections = 0
        End If

        ' Start this circle.
        X1 = e.X
        Y1 = e.Y
        X2 = e.X
        Y2 = e.Y
        Drawing = True
    End Sub
    Private Sub Form1_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
        If (Not Drawing) Then Return

        ' Update the new circle's second corner.
        X2 = e.X
        Y2 = e.Y

        ' Redraw.
        Me.Invalidate()
    End Sub
    Private Sub Form1_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        Drawing = False

        ' Save the new circle.
        Dim wid = Math.Min(Math.Abs(X1 - X2), Math.Abs(Y1 - Y2))
        Dim rect As New RectangleF(X1, Y1, wid, wid)
        If (X1 > X2) Then rect.X = X1 - wid
        If (Y1 > Y2) Then rect.Y = Y1 - wid
        Circles.Add(rect)

        ' If we have two circles, see where they intersect.
        If (Circles.Count = 2) Then
            Dim radius0 As Single = Circles(0).Height / 2
            Dim cx0 As Single = Circles(0).X + radius0
            Dim cy0 As Single = Circles(0).Y + radius0
            Dim radius1 As Single = Circles(1).Height / 2
            Dim cx1 As Single = Circles(1).X + radius1
            Dim cy1 As Single = Circles(1).Y + radius1
            NumIntersections = FindCircleCircleIntersections( _
                cx0, cy0, radius0, cx1, cy1, radius1, _
                Intersection1, Intersection2)
        End If

        ' Redraw.
        Me.Invalidate()
    End Sub

    ' Draw whatever we have.
    Private Sub Form1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias

        ' Draw the circles.
        For Each rect As RectangleF In Circles
            e.Graphics.DrawEllipse(Pens.Blue, rect)
        Next

        ' Draw the new circle.
        If (Drawing) Then
            Dim wid = Math.Min(Math.Abs(X1 - X2), Math.Abs(Y1 - Y2))
            Dim rect As New RectangleF(X1, Y1, wid, wid)
            If (X1 > X2) Then rect.X = X1 - wid
            If (Y1 > Y2) Then rect.Y = Y1 - wid
            e.Graphics.DrawEllipse(Pens.Red, rect)
        End If

        ' If we have a solution, display it.
        If (NumIntersections >= 1) Then
            DrawPoint(e.Graphics, Intersection1, Brushes.HotPink, Pens.Red)
        End If
        If (NumIntersections >= 2) Then
            DrawPoint(e.Graphics, Intersection2, Brushes.HotPink, Pens.Red)
        End If
    End Sub

    ' Find the points where the two circles intersect.
    Private Function FindCircleCircleIntersections( _
     ByVal cx0 As Single, ByVal cy0 As Single, ByVal radius0 As Single, _
     ByVal cx1 As Single, ByVal cy1 As Single, ByVal radius1 As Single, _
     ByRef intersection1 As PointF, ByRef intersection2 As PointF) As Integer
        ' Find the distance between the centers.
        Dim dx As Single = cx0 - cx1
        Dim dy As Single = cy0 - cy1
        Dim dist As Double = Math.Sqrt(dx * dx + dy * dy)

        ' See how many solutions there are.
        If (dist > radius0 + radius1) Then
            ' No solutions, the circles are too far apart.
            intersection1 = New PointF(Single.NaN, Single.NaN)
            intersection2 = New PointF(Single.NaN, Single.NaN)
            Return 0
        ElseIf (dist < Math.Abs(radius0 - radius1)) Then
            ' No solutions, one circle contains the other.
            intersection1 = New PointF(Single.NaN, Single.NaN)
            intersection2 = New PointF(Single.NaN, Single.NaN)
            Return 0
        ElseIf ((dist = 0) AndAlso (radius0 = radius1)) Then
            ' No solutions, the circles coincide.
            intersection1 = New PointF(Single.NaN, Single.NaN)
            intersection2 = New PointF(Single.NaN, Single.NaN)
            Return 0
        Else
            ' Find a and h.
            Dim a As Double = (radius0 * radius0 - _
                    radius1 * radius1 + dist * dist) / (2 * dist)
            Dim h As Double = Math.Sqrt(radius0 * radius0 - a * a)

            ' Find P2.
            Dim cx2 As Double = cx0 + a * (cx1 - cx0) / dist
            Dim cy2 As Double = cy0 + a * (cy1 - cy0) / dist

            ' Get the points P3.
            intersection1 = New PointF( _
                CSng(cx2 + h * (cy1 - cy0) / dist), _
                CSng(cy2 - h * (cx1 - cx0) / dist))
            intersection2 = New PointF( _
                CSng(cx2 - h * (cy1 - cy0) / dist), _
                CSng(cy2 + h * (cx1 - cx0) / dist))

            ' See if we have 1 or 2 solutions.
            If (dist = radius0 + radius1) Then Return 1
            Return 2
        End If
    End Function

    ' Draw a point.
    Private Sub DrawPoint(ByVal gr As Graphics, ByVal pt As PointF, ByVal brush As Brush, ByVal pen As Pen)
        Const RADIUS As Integer = 3
        gr.FillEllipse(brush, _
            pt.X - RADIUS, pt.Y - RADIUS, _
            2 * RADIUS, 2 * RADIUS)
        gr.DrawEllipse(pen, _
            pt.X - RADIUS, pt.Y - RADIUS, _
            2 * RADIUS, 2 * RADIUS)
    End Sub
End Class
